@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.4.9  Copyright by Tobi Oetiker, 1997-2014
+ * RRDtool 1.5.4, Copyright 1997-2015 by Tobi Oetiker
  *****************************************************************************
  * rrd_dump  Display a RRD
  *****************************************************************************
@@ -44,8 +44,8 @@
 #include "rrd_tool.h"
 #include "rrd_rpncalc.h"
 #include "rrd_client.h"
+#include "rrd_snprintf.h"
 
-#include <locale.h>
 
 #if !(defined(NETWARE) || defined(WIN32))
 extern char *tzname[2];
@@ -77,15 +77,19 @@ int rrd_dump_cb_r(
     rrd_t     rrd;
     rrd_value_t value;
     struct tm tm;
-    char *old_locale = "";
 
 //These two macros are local defines to clean up visible code from its redndancy
 //and make it easier to read.
 #define CB_PUTS(str)                                            \
-    cb((str), strlen((str)), user)
+    do {							\
+        size_t len = strlen(str);				\
+								\
+        if (cb((str), len, user) != len)                        \
+            goto err_out;                                       \
+    } while (0);
 #define CB_FMTS(...) do {                                       \
     char buffer[256];                                           \
-    snprintf (buffer, sizeof(buffer), __VA_ARGS__);             \
+    rrd_snprintf (buffer, sizeof(buffer), __VA_ARGS__);         \
     CB_PUTS (buffer);                                           \
     } while (0)
 //These macros are to be undefined at the end of this function
@@ -103,8 +107,6 @@ int rrd_dump_cb_r(
         return (-1);
     }
 
-    old_locale = setlocale(LC_NUMERIC, "C");
-
     if (opt_header == 1) {
         CB_PUTS("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
         CB_PUTS("<!DOCTYPE rrd SYSTEM \"http://oss.oetiker.ch/rrdtool/rrdtool.dtd\">\n");
@@ -125,7 +127,7 @@ int rrd_dump_cb_r(
     if (atoi(rrd.stat_head->version) <= 3) {
         CB_FMTS("\t<version>%s</version>\n", RRD_VERSION3);
     } else {
-        CB_FMTS("\t<version>%s</version>\n", RRD_VERSION);
+        CB_FMTS("\t<version>%s</version>\n", rrd.stat_head->version);
     }
     
     CB_FMTS("\t<step>%lu</step> <!-- Seconds -->\n",
@@ -138,7 +140,7 @@ int rrd_dump_cb_r(
 # error "Need strftime"
 #endif
     CB_FMTS("\t<lastupdate>%lld</lastupdate> <!-- %s -->\n\n",
-        (long long) rrd.live_head->last_up, somestring);
+        (long long int) rrd.live_head->last_up, somestring);
     for (i = 0; i < rrd.stat_head->ds_cnt; i++) {
         CB_PUTS("\t<ds>\n");
 
@@ -429,7 +431,7 @@ int rrd_dump_cb_r(
 #else
 # error "Need strftime"
 #endif
-            CB_FMTS("\t\t\t<!-- %s / %lld --> <row>",  somestring, (long long) now);
+            CB_FMTS("\t\t\t<!-- %s / %lld --> <row>",  somestring, (long long int) now);
             for (iii = 0; iii < rrd.stat_head->ds_cnt; iii++) {
                 rrd_read(rrd_file, &my_cdp, sizeof(rrd_value_t) * 1);
                 if (isnan(my_cdp)) {
@@ -447,10 +449,14 @@ int rrd_dump_cb_r(
 
     rrd_free(&rrd);
 
-    setlocale(LC_NUMERIC, old_locale);
-
     return rrd_close(rrd_file);
 
+err_out:
+    rrd_set_error("error writing output file: %s", rrd_strerror(errno));
+    rrd_free(&rrd);
+    rrd_close(rrd_file);
+    return (-1);
+
 //Undefining the previously defined shortcuts
 //See start of this function
 #undef CB_PUTS
@@ -486,8 +492,14 @@ int rrd_dump_opt_r(
 
     res = rrd_dump_cb_r(filename, opt_noheader, rrd_dump_opt_cb_fileout, (void *)out_file);
 
+    if (fflush(out_file) != 0) {
+        rrd_set_error("error flushing output: %s", rrd_strerror(errno));
+        res = -1;
+    }
     if (out_file != stdout) {
         fclose(out_file);
+        if (res != 0)
+            unlink(outname);
     }
 
     return res;
@@ -561,7 +573,9 @@ int rrd_dump(
 	   break;
 
         default:
-            rrd_set_error("usage rrdtool %s [--header|-h {none,xsd,dtd}] [--no-header]"
+            rrd_set_error("usage rrdtool %s [--header|-h {none,xsd,dtd}]\n"
+                          "[--no-header|-n]\n"
+                          "[--daemon|-d address]\n"
                           "file.rrd [file.xml]", argv[0]);
             return (-1);
             break;
@@ -569,8 +583,10 @@ int rrd_dump(
     }                   /* while (42) */
 
     if ((argc - optind) < 1 || (argc - optind) > 2) {
-        rrd_set_error("usage rrdtool %s [--header|-h {none,xsd,dtd}] [--no-header]"
-                      "file.rrd [file.xml]", argv[0]);
+        rrd_set_error("usage rrdtool %s [--header|-h {none,xsd,dtd}]\n"
+                      "[--no-header|-n]\n"
+                      "[--daemon|-d address]\n"
+                       "file.rrd [file.xml]", argv[0]);
         return (-1);
     }
 
