@@ -1,51 +1,7 @@
 /*****************************************************************************
- * RRDtool 1.4.9  Copyright by Tobi Oetiker, 1997-2014
+ * RRDtool 1.5.4, Copyright 1997-2015 by Tobi Oetiker
  *****************************************************************************
  * rrd_format.c  RRD Database Format helper functions
- *****************************************************************************
- * $Id$
- * $Log$
- * Revision 1.5  2004/05/18 18:53:03  oetiker
- * big spell checking patch -- slif@bellsouth.net
- *
- * Revision 1.4  2003/02/13 07:05:27  oetiker
- * Find attached the patch I promised to send to you. Please note that there
- * are three new source files (src/rrd_is_thread_safe.h, src/rrd_thread_safe.c
- * and src/rrd_not_thread_safe.c) and the introduction of librrd_th. This
- * library is identical to librrd, but it contains support code for per-thread
- * global variables currently used for error information only. This is similar
- * to how errno per-thread variables are implemented.  librrd_th must be linked
- * alongside of libpthred
- *
- * There is also a new file "THREADS", holding some documentation.
- *
- * -- Peter Stamfest <peter@stamfest.at>
- *
- * Revision 1.3  2002/02/01 20:34:49  oetiker
- * fixed version number and date/time
- *
- * Revision 1.2  2001/03/10 23:54:39  oetiker
- * Support for COMPUTE data sources (CDEF data sources). Removes the RPN
- * parser and calculator from rrd_graph and puts then in a new file,
- * rrd_rpncalc.c. Changes to core files rrd_create and rrd_update. Some
- * clean-up of aberrant behavior stuff, including a bug fix.
- * Documentation update (rrdcreate.pod, rrdupdate.pod). Change xml format.
- * -- Jake Brutlag <jakeb@corp.webtv.net>
- *
- * Revision 1.1.1.1  2001/02/25 22:25:05  oetiker
- * checkin
- *
- * Revision 1.3  1998/03/08 12:35:11  oetiker
- * checkpointing things because the current setup seems to work
- * according to the things said in the manpages
- *
- * Revision 1.2  1998/02/26 22:58:22  oetiker
- * fixed define
- *
- * Revision 1.1  1998/02/21 16:14:41  oetiker
- * Initial revision
- *
- *
  *****************************************************************************/
 #include "rrd_tool.h"
 #ifdef WIN32
@@ -57,13 +13,15 @@
 
 /* conversion functions to allow symbolic entry of enumerations */
 enum dst_en dst_conv(
-    char *string)
+    const char *string)
 {
     converter(COUNTER, DST_COUNTER)
         converter(ABSOLUTE, DST_ABSOLUTE)
         converter(GAUGE, DST_GAUGE)
         converter(DERIVE, DST_DERIVE)
         converter(COMPUTE, DST_CDEF)
+        converter(DCOUNTER, DST_DCOUNTER)
+        converter(DDERIVE, DST_DDERIVE)
         rrd_set_error("unknown data acquisition function '%s'", string);
     return (enum dst_en)(-1);
 }
@@ -87,6 +45,26 @@ enum cf_en cf_conv(
     return (enum cf_en)(-1);
 }
 
+const char *cf_to_string (enum cf_en cf)
+{
+    switch (cf)
+    {
+        case CF_AVERAGE:     return "AVERAGE";
+        case CF_MINIMUM:     return "MIN";
+        case CF_MAXIMUM:     return "MAX";
+        case CF_LAST:        return "LAST";
+        case CF_HWPREDICT:   return "HWPREDICT";
+        case CF_SEASONAL:    return "SEASONAL";
+        case CF_DEVPREDICT:  return "DEVPREDICT";
+        case CF_DEVSEASONAL: return "DEVSEASONAL";
+        case CF_FAILURES:    return "FAILURES";
+        case CF_MHWPREDICT:  return "MHWPREDICT";
+
+        default:
+            return NULL;
+    }
+} /* char *cf_to_string */
+
 #undef converter
 
 long ds_match(
