@@ -1,5 +1,5 @@
-#ifndef _RRD_GRAPH_H
-#define _RRD_GRAPH_H
+#ifndef RRD_GRAPH_H_DBEDBFB6C5844ED9BEA6242F879CA284
+#define RRD_GRAPH_H_DBEDBFB6C5844ED9BEA6242F879CA284
 
 #define y0 cairo_y0
 #define y1 cairo_y1
@@ -7,29 +7,28 @@
 
 /* this may configure __EXTENSIONS__ without which pango will fail to compile
    so load this early */
-#if defined(_WIN32) && !defined(__CYGWIN__) && !defined(__CYGWIN32__)
-#include "../win32/config.h"
-#else
-#ifdef HAVE_CONFIG_H
-#include "../rrd_config.h"
-#endif
-#endif
+#include "rrd_config.h"
 
 #include <cairo.h>
+#ifdef CAIRO_HAS_PDF_SURFACE
 #include <cairo-pdf.h>
+#endif
+#ifdef CAIRO_HAS_SVG_SURFACE
 #include <cairo-svg.h>
+#endif
+#ifdef CAIRO_HAS_PS_SURFACE
 #include <cairo-ps.h>
+#endif
 
 #include <pango/pangocairo.h>
 
 
 #include "rrd_tool.h"
 #include "rrd_rpncalc.h"
+#include "mutex.h"
+
+#include <glib.h>
 
-#ifdef WIN32
-#  include <windows.h>
-#  define MAXPATH MAX_PATH
-#endif
 
 #define ALTYGRID  	 0x01   /* use alternative y grid algorithm */
 #define ALTAUTOSCALE	 0x02   /* use alternative algorithm to find lower and upper bounds */
@@ -45,6 +44,7 @@
 
 #define FULL_SIZE_MODE     0x200    /* -width and -height indicate the total size of the image */
 #define NO_RRDTOOL_TAG 0x400  /* disable the rrdtool tag */
+#define ALLOW_MISSING_DS 0x800  /* missing DS is not fatal */
 
 #define gdes_fetch_key(x)  sprintf_alloc("%s:%d:%d:%d:%d",x.rrd,x.cf,x.cf_reduce,x.start_orig,x.end_orig,x.step_orig)
 
@@ -60,9 +60,9 @@ enum grc_en { GRC_CANVAS = 0, GRC_BACK, GRC_SHADEA, GRC_SHADEB,
 #define GRIDWIDTH  0.4
 
 enum gf_en { GF_PRINT = 0, GF_GPRINT, GF_COMMENT, GF_HRULE, GF_VRULE, GF_LINE,
-    GF_AREA, GF_STACK, GF_TICK, GF_TEXTALIGN,
+    GF_AREA,GF_STACK, GF_TICK, GF_TEXTALIGN,
     GF_DEF, GF_CDEF, GF_VDEF, GF_SHIFT,
-    GF_XPORT
+    GF_XPORT, GF_XAXIS, GF_YAXIS
 };
 
 enum txa_en { TXA_LEFT = 0, TXA_RIGHT, TXA_CENTER, TXA_JUSTIFIED };
@@ -81,7 +81,7 @@ enum vdef_op_en {
         , VDEF_LSLCORREL    /* least squares line correlation coefficient */
         , VDEF_PERCENTNAN  /* Nth percentile ignoring NAN*/
 };
-enum text_prop_en { 
+enum text_prop_en {
     TEXT_PROP_DEFAULT = 0,  /* default settings */
     TEXT_PROP_TITLE,    /* properties for the title */
     TEXT_PROP_AXIS,     /* for the numbers next to the axis */
@@ -92,13 +92,18 @@ enum text_prop_en {
 };
 
 enum legend_pos{ NORTH = 0, WEST, SOUTH, EAST };
-enum legend_direction { TOP_DOWN = 0, BOTTOM_UP };
+enum legend_direction { TOP_DOWN = 0, BOTTOM_UP, BOTTOM_UP2 };
 
-enum gfx_if_en { IF_PNG = 0, IF_SVG, IF_EPS, IF_PDF };
+enum gfx_if_en { IF_PNG = 0, IF_SVG, IF_EPS, IF_PDF,
+		 IF_XML=128, IF_CSV=129, IF_TSV=130, IF_SSV=131, IF_JSON=132,
+		 IF_XMLENUM=133, IF_JSONTIME=134
+};
 enum gfx_en { GFX_LINE = 0, GFX_AREA, GFX_TEXT };
 enum gfx_h_align_en { GFX_H_NULL = 0, GFX_H_LEFT, GFX_H_RIGHT, GFX_H_CENTER };
 enum gfx_v_align_en { GFX_V_NULL = 0, GFX_V_TOP, GFX_V_BOTTOM, GFX_V_CENTER };
 
+enum gfx_type_en {GTYPE_TIME=0,GTYPE_XY=1};
+
 /* cairo color components */
 typedef struct gfx_color_t {
     double    red;
@@ -107,6 +112,39 @@ typedef struct gfx_color_t {
     double    alpha;
 } gfx_color_t;
 
+typedef struct keyvalue_t {
+  char* key;
+  char* value;
+  char* keyvalue;
+  int pos;
+  int flag;
+} keyvalue_t;
+
+typedef struct keyint_t {
+  char*key;
+  int value;
+} keyint_t;
+
+typedef struct parsedargs_t {
+  char *arg; /* copy of the parsed string - rewritten*/
+  const char *arg_orig; /* original argument */
+  int kv_cnt; /* number of key/value arguments */
+  keyvalue_t *kv_args; /* key value arguments */
+} parsedargs_t;
+void initParsedArguments(parsedargs_t*);
+void resetParsedArguments(parsedargs_t*);
+void freeParsedArguments(parsedargs_t*);
+int addToArguments(parsedargs_t*, char*, char*, char*, int);
+int parseArguments(const char*, parsedargs_t*);
+void dumpKeyValue(char* ,keyvalue_t*);
+void dumpArguments(parsedargs_t*);
+char* getKeyValueArgument(const char*, int, parsedargs_t*);
+int getMappedKeyValueArgument(const char*,int, parsedargs_t*,
+			      int*,keyint_t**);
+int getLong(const char*,long *,char**,int);
+int getDouble(const char*,double *,char**);
+keyvalue_t* getFirstUnusedArgument(int, parsedargs_t*);
+char* checkUnusedValues(parsedargs_t*);
 
 typedef struct text_prop_t {
     double    size;
@@ -120,10 +158,11 @@ typedef struct vdef_t {
     double    param;    /* parameter for function, if applicable */
     double    val;      /* resulting value */
     time_t    when;     /* timestamp, if applicable */
+    int       never;    /* boolean, indicate that when value mean never */
 } vdef_t;
 
 typedef struct xlab_t {
-    long      minsec;   /* minimum sec per pix */
+    double    minsec;   /* minimum sec per pix */
     long      length;   /* number of secs on the image */
     enum tmt_en gridtm; /* grid interval in what ? */
     long      gridst;   /* how many whats per grid */
@@ -148,6 +187,12 @@ typedef struct ylab_t {
     int       lfac[4];  /* associated label spacing */
 } ylab_t;
 
+enum value_formatter_en {
+    VALUE_FORMATTER_NUMERIC,    /* printf */
+    VALUE_FORMATTER_TIMESTAMP,  /* strftime */
+    VALUE_FORMATTER_DURATION,   /* strfduration */
+};
+
 /* this structure describes the elements which can make up a graph.
    because they are quite diverse, not all elements will use all the
    possible parts of the structure. */
@@ -157,27 +202,34 @@ typedef struct ylab_t {
 #define FMT_LEG_LEN 2000
 #endif
 
+# define MAX_AXIS 4
+
 typedef struct graph_desc_t {
-    enum gf_en gf;      /* graphing function */
-    int       stack;    /* boolean */
-    int       debug;    /* boolean */
+    enum gf_en gf;       /* graphing function */
+    int       stack;     /* boolean */
+    int       debug;     /* boolean */
     int       skipscale; /* boolean */
     char      vname[MAX_VNAME_LEN + 1]; /* name of the variable */
-    long      vidx;     /* gdes reference */
+    long      vidx;      /* gdes reference */
     char      rrd[1024];    /* name of the rrd_file containing data */
     char      ds_nam[DS_NAM_SIZE];  /* data source name */
     long      ds;       /* data source number */
+    char      daemon[256];
     enum cf_en cf;      /* consolidation function */
     enum cf_en cf_reduce;   /* consolidation function for reduce_data() */
-    struct gfx_color_t col; /* graph color */
+    int        cf_reduce_set; /* is the cf_reduce option set */
+    struct gfx_color_t col, col2; /* graph color */
+	double    gradheight;
     char      format[FMT_LEG_LEN + 5];  /* format for PRINT AND GPRINT */
     char      legend[FMT_LEG_LEN + 5];  /* legend */
-    int       strftm;   /* should the VDEF legend be formated with strftime */
+    int       strftm;   /* should the VDEF legend be the time component formated with strftime */
+    enum value_formatter_en vformatter; /* what value formatter to use (if !strftm) */
     double    leg_x, leg_y; /* location of legend */
     double    yrule;    /* value for y rule line and for VDEF */
     time_t    xrule;    /* time for x rule line and for VDEF */
     vdef_t    vf;       /* instruction for VDEF function */
     rpnp_t   *rpnp;     /* instructions for CDEF function */
+    char     *rpn;      /* string representation of rpn */
 
     /* SHIFT implementation */
     int       shidx;    /* gdes reference for offset (-1 --> constant) */
@@ -204,43 +256,49 @@ typedef struct graph_desc_t {
 
 
     enum txa_en txtalign;   /* change default alignment strategy for text */
+
+    /* the axis to use for this graph in x and y*/
+    int xaxisidx;
+    int yaxisidx;
 } graph_desc_t;
 
 typedef struct image_desc_t {
 
     /* configuration of graph */
-
-    char      graphfile[MAXPATH];   /* filename for graphic */
+    char      *graphfile;   /* filename for graphic */
+    enum      gfx_type_en graph_type; /* type of the graph */
     long      xsize, ysize; /* graph area size in pixels */
     struct gfx_color_t graph_col[__GRC_END__];  /* real colors for the graph */
     text_prop_t text_prop[TEXT_PROP_LAST];  /* text properties */
-    char      ylegend[210]; /* legend along the yaxis */
-    char      title[210];   /* title for graph */
-    char      watermark[110];   /* watermark for graph */
+    char      *ylegend; /* legend along the yaxis */
+    char      *title;   /* title for graph */
+    char      *watermark;   /* watermark for graph */
     int       draw_x_grid;  /* no x-grid at all */
     int       draw_y_grid;  /* no y-grid at all */
     unsigned int draw_3d_border; /* size of border in pixels, 0 for off */
     unsigned int dynamic_labels; /* pick the label shape according to the line drawn */
     double    grid_dash_on, grid_dash_off;
     xlab_t    xlab_user;    /* user defined labeling for xaxis */
-    char      xlab_form[210];   /* format for the label on the xaxis */
+    char      *xlab_form;   /* format for the label on the xaxis */
     double    second_axis_scale; /* relative to the first axis (0 to disable) */
     double    second_axis_shift; /* how much is it shifted vs the first axis */
-    char      second_axis_legend[210]; /* label to put on the seond axis */
-    char      second_axis_format[210]; /* format for the numbers on the scond axis */    
-    char      primary_axis_format[210]; /* format for the numbers on the primary axis */
+    char      *second_axis_legend; /* label to put on the seond axis */
+    char      *second_axis_format; /* format for the numbers on the scond axis */
+    enum value_formatter_en second_axis_formatter;  /* How to format axis values */
+    char      *primary_axis_format; /* format for the numbers on the primary axis */
+    enum value_formatter_en primary_axis_formatter; /* How to format axis values */
     double    ygridstep;    /* user defined step for y grid */
     int       ylabfact; /* every how many y grid shall a label be written ? */
     double    tabwidth; /* tabwdith */
     time_t    start, end;   /* what time does the graph cover */
     unsigned long step; /* any preference for the default step ? */
     rrd_value_t minval, maxval; /* extreme values in the data */
-    int       rigid;    /* do not expand range even with 
+    int       rigid;    /* do not expand range even with
                            values outside */
     ygrid_scale_t ygrid_scale;  /* calculated y axis grid info */
     int       gridfit;  /* adjust y-axis range etc so all
                            grindlines falls in integer pixel values */
-    char     *imginfo;  /* construct an <IMG ... tag and return 
+    char     *imginfo;  /* construct an <IMG ... tag and return
                            as first retval */
     enum gfx_if_en imgformat;   /* image format */
     char     *daemon_addr;  /* rrdcached connection string */
@@ -289,6 +347,7 @@ typedef struct image_desc_t {
     rrd_info_t *grinfo_current; /* pointing to current entry */
     GHashTable* gdef_map;  /* a map of all *def gdef entries for quick access */
     GHashTable* rrd_map;  /* a map of all rrd files in use for gdef entries */
+    mutex_t *fontmap_mutex; /* Mutex for locking the global fontmap */
 } image_desc_t;
 
 /* Prototypes */
@@ -302,6 +361,8 @@ enum gf_en gf_conv(
     char *);
 enum gfx_if_en if_conv(
     char *);
+enum gfx_type_en type_conv(
+    char *);
 enum tmt_en tmt_conv(
     char *);
 enum grc_en grc_conv(
@@ -331,12 +392,6 @@ void      reduce_data(
     rrd_value_t **);
 int       data_fetch(
     image_desc_t *);
-long      find_var(
-    image_desc_t *,
-    char *);
-long      find_var_wrapper(
-    void *arg1,
-    char *key);
 long      lcd(
     long *);
 int       data_calc(
@@ -366,12 +421,23 @@ void      vertical_grid(
     image_desc_t *);
 void      axis_paint(
     image_desc_t *);
-void      grid_paint(
+int      grid_paint(
     image_desc_t *);
 int       lazy_check(
     image_desc_t *);
 int       graph_paint(
     image_desc_t *);
+int       graph_paint_timestring(
+                                image_desc_t *,int,int);
+int       graph_paint_xy(
+                        image_desc_t *,int,int);
+int       rrd_graph_xport(
+    image_desc_t *);
+
+int       graph_cairo_setup(
+    image_desc_t *);
+int       graph_cairo_finish(
+    image_desc_t *);
 
 int       gdes_alloc(
     image_desc_t *);
@@ -382,6 +448,10 @@ int       scan_for_col(
 void      rrd_graph_init(
     image_desc_t *);
 
+void      time_clean(
+    char *result,
+    char *format);
+
 void      rrd_graph_options(
     int,
     char **,
@@ -454,6 +524,25 @@ void      gfx_add_point(
     double x,
     double y);
 
+/* create a rect that has a gradient from color1 to color2 in height pixels
+ * height > 0:
+ * 		gradient starts at top and goes down a fixed number of pixels (fire style)
+ * height < 0:
+ * 		gradient starts at bottom and goes up a fixed number of pixels (constant style)
+ * height == 0:
+ * 		gradient is stretched between two points
+ */
+void gfx_add_rect_fadey(
+    image_desc_t *im,
+    double x1,double y1,
+    double x2,double y2,
+	double py,
+    gfx_color_t color1,
+	gfx_color_t color2,
+	double height);
+
+
+
 /* close current path so it ends at the same point as it started */
 void      gfx_close_path(
     image_desc_t *im);
@@ -501,3 +590,5 @@ void      grinfo_push(
     image_desc_t *im,
     char *key,
     rrd_info_type_t type,    rrd_infoval_t value);
+
+
