@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.4.9  Copyright by Tobi Oetiker, 1997-2014
+ * RRDtool 1.5.4, Copyright 1997-2015 by Tobi Oetiker
  *****************************************************************************
  * rrd_info  Get Information about the configuration of an RRD
  *****************************************************************************/
@@ -14,7 +14,7 @@ rrd_info_t *rrd_info(
     int,
     char **);
 rrd_info_t *rrd_info_r(
-    char *filename);
+    const char *filename);
 
 /* allocate memory for string */
 char     *sprintf_alloc(
@@ -71,8 +71,7 @@ rrd_info_t
         next->value.u_int = value.u_int;
         break;
     case RD_I_STR:
-        next->value.u_str = (char*)malloc(sizeof(char) * (strlen(value.u_str) + 1));
-        strcpy(next->value.u_str, value.u_str);
+        next->value.u_str = strdup(value.u_str);
         break;
     case RD_I_BLO:
         next->value.u_blo.size = value.u_blo.size;
@@ -92,6 +91,7 @@ rrd_info_t *rrd_info(
     rrd_info_t *info;
     char *opt_daemon = NULL;
     int status;
+    int flushfirst = 1;
 
     optind = 0;
     opterr = 0;         /* initialize getopt */
@@ -101,10 +101,11 @@ rrd_info_t *rrd_info(
         int       option_index = 0;
         static struct option long_options[] = {
             {"daemon", required_argument, 0, 'd'},
+            {"noflush", no_argument, 0, 'F'},
             {0, 0, 0, 0}
         };
 
-        opt = getopt_long(argc, argv, "d:", long_options, &option_index);
+        opt = getopt_long(argc, argv, "d:F", long_options, &option_index);
 
         if (opt == EOF)
             break;
@@ -121,8 +122,12 @@ rrd_info_t *rrd_info(
             }
             break;
 
+        case 'F':
+            flushfirst = 0;
+            break;
+
         default:
-            rrd_set_error ("Usage: rrdtool %s [--daemon <addr>] <file>",
+            rrd_set_error ("Usage: rrdtool %s [--daemon|-d <addr> [--noflush|-F]] <file>",
                     argv[0]);
             return (NULL);
             break;
@@ -130,22 +135,28 @@ rrd_info_t *rrd_info(
     }                   /* while (42) */
 
     if ((argc - optind) != 1) {
-        rrd_set_error ("Usage: rrdtool %s [--daemon <addr>] <file>",
+        rrd_set_error ("Usage: rrdtool %s [--daemon |-d <addr> [--noflush|-F]] <file>",
                 argv[0]);
         return (NULL);
     }
 
+    if( flushfirst ) {
     status = rrdc_flush_if_daemon(opt_daemon, argv[optind]);
-    if (opt_daemon) free (opt_daemon);
     if (status) return (NULL);
+    }
 
+    rrdc_connect (opt_daemon);
+    if (rrdc_is_connected (opt_daemon))
+        info = rrdc_info (argv[optind]);
+    else
     info = rrd_info_r(argv[optind]);
 
+    if (opt_daemon) free(opt_daemon);
     return (info);
 } /* rrd_info_t *rrd_info */
 
 rrd_info_t *rrd_info_r(
-    char *filename)
+    const char *filename)
 {
     unsigned int i, ii = 0;
     rrd_t     rrd;
