@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.4.9  Copyright by Tobi Oetiker, 1997-2014
+ * RRDtool 1.5.4, Copyright 1997-2015 by Tobi Oetiker
  *****************************************************************************
  * rrd_last.c
  *****************************************************************************
@@ -14,7 +14,7 @@ time_t rrd_last(
     char **argv)
 {
     char *opt_daemon = NULL;
-    int status;
+    time_t lastupdate;
 
     optind = 0;
     opterr = 0;         /* initialize getopt */
@@ -45,7 +45,7 @@ time_t rrd_last(
             break;
 
         default:
-            rrd_set_error ("Usage: rrdtool %s [--daemon <addr>] <file>",
+            rrd_set_error ("Usage: rrdtool %s [--daemon|-d <addr>] <file>",
                     argv[0]);
             return (-1);
             break;
@@ -53,16 +53,20 @@ time_t rrd_last(
     }                   /* while (42) */
 
     if ((argc - optind) != 1) {
-        rrd_set_error ("Usage: rrdtool %s [--daemon <addr>] <file>",
+        rrd_set_error ("Usage: rrdtool %s [--daemon|-d <addr>] <file>",
                 argv[0]);
         return (-1);
     }
 
-    status = rrdc_flush_if_daemon(opt_daemon, argv[optind]);
-    if (opt_daemon) free(opt_daemon);
-    if (status) return (-1);
+    rrdc_connect (opt_daemon);
+    if (rrdc_is_connected (opt_daemon))
+        lastupdate = rrdc_last (argv[optind]);
 
-    return (rrd_last_r (argv[optind]));
+    else
+        lastupdate = rrd_last_r(argv[optind]);
+
+    if (opt_daemon) free(opt_daemon);
+    return (lastupdate);
 }
 
 time_t rrd_last_r(
