@@ -1,5 +1,5 @@
 /****************************************************************************
- * RRDtool 1.4.9  Copyright by Tobi Oetiker, 1997-2014
+ * RRDtool 1.5.4, Copyright 1997-2015 by Tobi Oetiker
  ****************************************************************************
  * rrd_rpncalc.c  RPN calculator functions
  ****************************************************************************/
@@ -7,7 +7,13 @@
 #include <limits.h>
 #include <locale.h>
 #include <stdlib.h>
+#include <time.h>
+#include "rrd_tool.h"
 
+#ifdef HAVE_LANGINFO_H
+#include <langinfo.h>
+#endif
+#include "rrd_strtod.h"
 #include "rrd_tool.h"
 #include "rrd_rpncalc.h"
 // #include "rrd_graph.h"
@@ -78,6 +84,8 @@ rpnp_t   *rpn_expand(
     }
     for (i = 0; rpnc[i].op != OP_END; ++i) {
         rpnp[i].op = (enum op_en)rpnc[i].op;
+	rpnp[i].extra = NULL;
+	rpnp[i].free_extra = NULL;
         if (rpnp[i].op == OP_NUMBER) {
             rpnp[i].val = (double) rpnc[i].val;
         } else if (rpnp[i].op == OP_VARIABLE || rpnp[i].op == OP_PREV_OTHER) {
@@ -117,7 +125,7 @@ void rpn_compact2str(
 #if defined(_WIN32) && !defined(__CYGWIN__) && !defined(__CYGWIN32__)
             _itoa(rpnc[i].val, buffer, 10);
 #else
-            sprintf(buffer, "%d", rpnc[i].val);
+            snprintf(buffer, sizeof buffer, "%d", rpnc[i].val);
 #endif
             add_op(OP_NUMBER, buffer)
         }
@@ -170,6 +178,11 @@ void rpn_compact2str(
             add_op(OP_ISINF, ISINF)
             add_op(OP_NOW, NOW)
             add_op(OP_LTIME, LTIME)
+            add_op(OP_NEWDAY, NEWDAY)
+            add_op(OP_NEWWEEK, NEWWEEK)
+            add_op(OP_NEWMONTH, NEWMONTH)
+            add_op(OP_NEWYEAR, NEWYEAR)
+            add_op(OP_STEPWIDTH, STEPWIDTH)
             add_op(OP_TIME, TIME)
             add_op(OP_ATAN2, ATAN2)
             add_op(OP_ATAN, ATAN)
@@ -180,6 +193,7 @@ void rpn_compact2str(
             add_op(OP_TRENDNAN, TRENDNAN)
             add_op(OP_PREDICT, PREDICT)
             add_op(OP_PREDICTSIGMA, PREDICTSIGMA)
+            add_op(OP_PREDICTPERC, PREDICTPERC)
             add_op(OP_RAD2DEG, RAD2DEG)
             add_op(OP_DEG2RAD, DEG2RAD)
             add_op(OP_AVG, AVG)
@@ -187,6 +201,10 @@ void rpn_compact2str(
             add_op(OP_ADDNAN, ADDNAN)
             add_op(OP_MINNAN, MINNAN)
             add_op(OP_MAXNAN, MAXNAN)
+            add_op(OP_DEPTH, DEPTH)
+            add_op(OP_COPY, COPY)
+            add_op(OP_ROLL, ROLL)
+            add_op(OP_INDEX, INDEX)
 #undef add_op
     }
     (*str)[offset] = '\0';
@@ -218,16 +236,17 @@ short addop2str(
     return 0;
 }
 
-void parseCDEF_DS(
-    const char *def,
-    rrd_t *rrd,
-    int ds_idx)
+void parseCDEF_DS(const char *def,
+		  ds_def_t *ds_def,
+		  void *key_hash,
+		  long (*lookup) (void *, char *)
+		  )
 {
     rpnp_t   *rpnp = NULL;
     rpn_cdefds_t *rpnc = NULL;
     short     count, i;
 
-    rpnp = rpn_parse((void *) rrd, def, &lookup_DS);
+    rpnp = rpn_parse(key_hash, def, lookup);
     if (rpnp == NULL) {
         rrd_set_error("failed to parse computed data source");
         return;
@@ -240,9 +259,20 @@ void parseCDEF_DS(
         if (rpnp[i].op == OP_TIME || rpnp[i].op == OP_LTIME ||
             rpnp[i].op == OP_PREV || rpnp[i].op == OP_COUNT ||
             rpnp[i].op == OP_TREND || rpnp[i].op == OP_TRENDNAN ||
-            rpnp[i].op == OP_PREDICT || rpnp[i].op ==  OP_PREDICTSIGMA ) {
+            rpnp[i].op == OP_PREDICT || rpnp[i].op ==  OP_PREDICTSIGMA ||
+            rpnp[i].op == OP_PREDICTPERC ||
+            /* these could actually go into COMPUTE with RRD format 06 ... since adding new
+               stuff into COMPUTE requires a fileformat update and that can only happen with the
+               1.6 release */
+            rpnp[i].op == OP_STEPWIDTH ||
+            rpnp[i].op == OP_NEWDAY ||
+            rpnp[i].op == OP_NEWWEEK ||
+            rpnp[i].op == OP_NEWMONTH ||
+            rpnp[i].op == OP_NEWYEAR
+        ) {
+
             rrd_set_error
-                ("operators TIME, LTIME, PREV COUNT TREND TRENDNAN PREDICT PREDICTSIGMA are not supported with DS COMPUTE");
+                ("operators TIME LTIME STEPWIDTH PREV NEW* COUNT TREND TRENDNAN PREDICT PREDICTSIGMA PREDICTPERC are not supported with DS COMPUTE");
             free(rpnp);
             return;
         }
@@ -252,7 +282,7 @@ void parseCDEF_DS(
         return;
     }
     /* copy the compact rpn representation over the ds_def par array */
-    memcpy((void *) &(rrd->ds_def[ds_idx].par[DS_cdef]),
+    memcpy((void *) &(ds_def->par[DS_cdef]),
            (void *) rpnc, count * sizeof(rpn_cdefds_t));
     free(rpnp);
     free(rpnc);
@@ -299,9 +329,7 @@ rpnp_t   *rpn_parse(
     long      steps = -1;
     rpnp_t   *rpnp;
     char      vname[MAX_VNAME_LEN + 10];
-    char     *old_locale;
-
-    old_locale = setlocale(LC_NUMERIC, "C");
+    char      double_str[41] = {0};
 
     rpnp = NULL;
     expr = (char *) expr_const;
@@ -309,12 +337,12 @@ rpnp_t   *rpn_parse(
     while (*expr) {
         if ((rpnp = (rpnp_t *) rrd_realloc(rpnp, (++steps + 2) *
                                            sizeof(rpnp_t))) == NULL) {
-            setlocale(LC_NUMERIC, old_locale);
             return NULL;
         }
 
-        else if ((sscanf(expr, "%lf%n", &rpnp[steps].val, &pos) == 1)
-                 && (expr[pos] == ',')) {
+        else if ((sscanf(expr, "%40[0-9.e+-]%n", double_str, &pos) == 1)
+                 && (expr[pos] == ',')
+                 && ( rrd_strtodbl( double_str, NULL, &(rpnp[steps].val), NULL ) == 2 )) {
             rpnp[steps].op = OP_NUMBER;
             expr += pos;
         }
@@ -354,6 +382,11 @@ rpnp_t   *rpn_parse(
             match_op(OP_EXC, EXC)
             match_op(OP_POP, POP)
             match_op(OP_LTIME, LTIME)
+            match_op(OP_NEWDAY, NEWDAY)
+            match_op(OP_NEWWEEK, NEWWEEK)
+            match_op(OP_NEWMONTH, NEWMONTH)
+            match_op(OP_NEWYEAR, NEWYEAR)
+            match_op(OP_STEPWIDTH, STEPWIDTH)
             match_op(OP_LT, LT)
             match_op(OP_LE, LE)
             match_op(OP_GT, GT)
@@ -384,6 +417,7 @@ rpnp_t   *rpn_parse(
             match_op(OP_TRENDNAN, TRENDNAN)
             match_op(OP_PREDICT, PREDICT)
             match_op(OP_PREDICTSIGMA, PREDICTSIGMA)
+            match_op(OP_PREDICTPERC, PREDICTPERC)
             match_op(OP_RAD2DEG, RAD2DEG)
             match_op(OP_DEG2RAD, DEG2RAD)
             match_op(OP_AVG, AVG)
@@ -391,6 +425,12 @@ rpnp_t   *rpn_parse(
             match_op(OP_ADDNAN, ADDNAN)
             match_op(OP_MINNAN, MINNAN)
             match_op(OP_MAXNAN, MAXNAN)
+            match_op(OP_MEDIAN, MEDIAN)
+            match_op(OP_DEPTH, DEPTH)
+            match_op(OP_COPY, COPY)
+            match_op(OP_ROLL, ROLL)
+            match_op(OP_INDEX, INDEX)
+
 #undef match_op
             else if ((sscanf(expr, DEF_NAM_FMT "%n", vname, &pos) == 1)
                      && ((rpnp[steps].ptr = (*lookup) (key_hash, vname)) !=
@@ -401,23 +441,23 @@ rpnp_t   *rpn_parse(
 
         else {
             rrd_set_error("don't undestand '%s'",expr);
-            setlocale(LC_NUMERIC, old_locale);
             free(rpnp);
             return NULL;
         }
 
+	rpnp[steps].extra = NULL;
+	rpnp[steps].free_extra = NULL;
+
         if (*expr == 0)
             break;
         if (*expr == ',')
             expr++;
         else {
-            setlocale(LC_NUMERIC, old_locale);
             free(rpnp);
             return NULL;
         }
     }
     rpnp[steps + 1].op = OP_END;
-    setlocale(LC_NUMERIC, old_locale);
     return rpnp;
 }
 
@@ -426,17 +466,34 @@ void rpnstack_init(
 {
     rpnstack->s = NULL;
     rpnstack->dc_stacksize = 0;
-    rpnstack->dc_stackblock = 100;
+    rpnstack->dc_stackblock = 1000;
 }
 
 void rpnstack_free(
     rpnstack_t *rpnstack)
 {
-    if (rpnstack->s != NULL)
-        free(rpnstack->s);
+    free(rpnstack->s);
     rpnstack->dc_stacksize = 0;
 }
 
+void rpnp_freeextra(rpnp_t* rpnp)
+{
+    int rpi;
+    if (!rpnp)
+      return;
+    /* process each op from the rpn in turn */
+    for (rpi = 0; rpnp[rpi].op != OP_END; rpi++) {
+        if (rpnp[rpi].extra) {
+	    if (rpnp[rpi].free_extra) {
+	        rpnp[rpi].free_extra(rpnp[rpi].extra);
+	    } else {
+	        free(rpnp[rpi].extra);
+	    }
+	    rpnp[rpi].extra = NULL;
+	}
+    }
+}
+
 static int rpn_compare_double(
     const void *x,
     const void *y)
@@ -446,8 +503,32 @@ static int rpn_compare_double(
     return (diff < 0) ? -1 : (diff > 0) ? 1 : 0;
 }
 
+static int find_first_weekday(void){
+    static int first_weekday = -1;
+    if (first_weekday == -1){
+#ifdef HAVE__NL_TIME_WEEK_1STDAY
+        /* according to http://sourceware.org/ml/libc-locales/2009-q1/msg00011.html */
+        /* See correct way here http://pasky.or.cz/dev/glibc/first_weekday.c */
+        first_weekday = nl_langinfo (_NL_TIME_FIRST_WEEKDAY)[0];
+        int week_1stday;
+        long week_1stday_l = (long) nl_langinfo (_NL_TIME_WEEK_1STDAY);
+        if (week_1stday_l == 19971130) week_1stday = 0; /* Sun */
+        else if (week_1stday_l == 19971201) week_1stday = 1; /* Mon */
+        else
+        {
+            first_weekday = 1;
+            return first_weekday; /* we go for a monday default */
+        }
+        first_weekday=(week_1stday + first_weekday - 1) % 7;
+#else
+        first_weekday = 1;
+#endif
+    }
+    return first_weekday;
+}
+
 /* rpn_calc: run the RPN calculator; also performs variable substitution;
- * moved and modified from data_calc() originally included in rrd_graph.c 
+ * moved and modified from data_calc() originally included in rrd_graph.c
  * arguments:
  * rpnp : an array of RPN operators (including variable references)
  * rpnstack : the initialized stack
@@ -466,11 +547,13 @@ short rpn_calc(
     rpnstack_t *rpnstack,
     long data_idx,
     rrd_value_t *output,
-    int output_idx)
-{
+    int output_idx,
+    int step_width
+){
     int       rpi;
     long      stptr = -1;
-
+    struct tm tmtmp1,tmtmp2;
+    time_t    timetmp;
     /* process each op from the rpn in turn */
     for (rpi = 0; rpnp[rpi].op != OP_END; rpi++) {
         /* allocate or grow the stack */
@@ -524,6 +607,9 @@ short rpn_calc(
                 }
             }
             break;
+        case OP_STEPWIDTH:
+            rpnstack->s[++stptr] = step_width;
+            break;
         case OP_COUNT:
             rpnstack->s[++stptr] = (output_idx + 1);    /* Note: Counter starts at 1 */
             break;
@@ -555,6 +641,34 @@ short rpn_calc(
             rpnstack->s[++stptr] =
                 (double) tzoffset(data_idx) + (double) data_idx;
             break;
+        case OP_NEWDAY:
+            timetmp = data_idx;
+            localtime_r(&timetmp,&tmtmp1);
+            timetmp = data_idx-step_width;
+            localtime_r(&timetmp,&tmtmp2);
+            rpnstack->s[++stptr] = tmtmp1.tm_mday != tmtmp2.tm_mday ? 1.0 : 0.0;
+            break;
+        case OP_NEWWEEK:
+            timetmp = data_idx;
+            localtime_r(&timetmp,&tmtmp1);
+            timetmp = data_idx-step_width;
+            localtime_r(&timetmp,&tmtmp2);
+            rpnstack->s[++stptr] = (tmtmp1.tm_wday == find_first_weekday() && tmtmp1.tm_wday != tmtmp2.tm_wday) ? 1.0 : 0.0;
+            break;
+        case OP_NEWMONTH:
+            timetmp = data_idx;
+            localtime_r(&timetmp,&tmtmp1);
+            timetmp = data_idx-step_width;
+            localtime_r(&timetmp,&tmtmp2);
+            rpnstack->s[++stptr] = tmtmp1.tm_mon != tmtmp2.tm_mon? 1.0 : 0.0;
+            break;
+        case OP_NEWYEAR:
+            timetmp = data_idx;
+            localtime_r(&timetmp,&tmtmp1);
+            timetmp = data_idx-step_width;
+            localtime_r(&timetmp,&tmtmp2);
+            rpnstack->s[++stptr] = tmtmp1.tm_year != tmtmp2.tm_year ? 1.0: 0.0;
+            break;
         case OP_ADD:
             stackunderflow(1);
             rpnstack->s[stptr - 1] = rpnstack->s[stptr - 1]
@@ -724,8 +838,7 @@ short rpn_calc(
             stackunderflow(2);
             rpnstack->s[stptr - 2] = (isnan(rpnstack->s[stptr - 2])
                                       || rpnstack->s[stptr - 2] ==
-                                      0.0) ? rpnstack->s[stptr] : rpnstack->
-                s[stptr - 1];
+                                      0.0) ? rpnstack->s[stptr] : rpnstack->s[stptr - 1];
             stptr--;
             stptr--;
             break;
@@ -820,12 +933,27 @@ short rpn_calc(
             break;
         case OP_PREDICT:
         case OP_PREDICTSIGMA:
-            stackunderflow(2);
+        case OP_PREDICTPERC:
 	    {
-		/* the local averaging window (similar to trend, but better here, as we get better statistics thru numbers)*/
+	        /* the percentile requested */
+	        double  percentile = DNAN;
+		if (rpnp[rpi].op == OP_PREDICTPERC) {
+		    stackunderflow(1);
+		    percentile = rpnstack->s[--stptr];
+		    if (abs(percentile) > 100) {
+		        rrd_set_error("unsupported percentile: %f",percentile);
+			return -1;
+		    }
+		    percentile/=100;
+		}
+		/* the local averaging window (similar to trend,
+		 * but better here, as we get better statistics
+		 * thru numbers)*/
+	        stackunderflow(2);
 		int   locstepsize = rpnstack->s[--stptr];
 		/* the number of shifts and range-checking*/
 		int     shifts = rpnstack->s[--stptr];
+
                 stackunderflow(shifts);
 		// handle negative shifts special
 		if (shifts<0) {
@@ -836,9 +964,9 @@ short rpn_calc(
 		/* the real calculation */
 		double val=DNAN;
 		/* the info on the datasource */
-		time_t  dsstep = (time_t) rpnp[rpi - 1].step;
-		int    dscount = rpnp[rpi - 1].ds_cnt;
-		int   locstep = (int)ceil((float)locstepsize/(float)dsstep);
+		time_t  dsstep  = (time_t) rpnp[rpi - 1].step;
+		int     dscount = rpnp[rpi - 1].ds_cnt;
+		int     locstep = (int)ceil((float)locstepsize/(float)dsstep);
 
 		/* the sums */
                 double    sum = 0;
@@ -847,13 +975,23 @@ short rpn_calc(
 		/* now loop for each position */
 		int doshifts=shifts;
 		if (shifts<0) { doshifts=-shifts; }
+		/* alloc memory */
+		double *extra = (double *) rpnp[rpi].extra;
+		if (rpnp[rpi].op == OP_PREDICTPERC) {
+		    if (! extra) {
+		      int size = (doshifts + 1) * (locstep + 2);
+		      rpnp[rpi].extra =
+			  extra =  (double *) malloc(sizeof(double) * size);
+		    }
+		}
+		/* loop the shifts */
 		for(int loop=0;loop<doshifts;loop++) {
 		    /* calculate shift step */
 		    int shiftstep=1;
 		    if (shifts<0) {
 			shiftstep = loop*rpnstack->s[stptr];
-		    } else { 
-			shiftstep = rpnstack->s[stptr+loop]; 
+		    } else {
+			shiftstep = rpnstack->s[stptr+loop];
 		    }
 		    if(shiftstep <0) {
 			rrd_set_error("negative shift step not allowed: %i",shiftstep);
@@ -862,7 +1000,8 @@ short rpn_calc(
 		    shiftstep=(int)ceil((float)shiftstep/(float)dsstep);
 		    /* loop all local shifts */
 		    for(int i=0;i<=locstep;i++) {
-			/* now calculate offset into data-array - relative to output_idx*/
+			/* now calculate offset into data-array
+			 * - relative to output_idx */
 			int offset=shiftstep+i;
 			/* and process if we have index 0 of above */
 			if ((offset>=0)&&(offset<output_idx)) {
@@ -872,6 +1011,9 @@ short rpn_calc(
 			    if (! isnan(val)) {
 				sum+=val;
 				sum2+=val*val;
+				if (extra) {
+				    extra[count]=val;
+				}
 				count++;
 			    }
 			}
@@ -879,11 +1021,13 @@ short rpn_calc(
 		}
 		/* do the final calculations */
 		val=DNAN;
-		if (rpnp[rpi].op == OP_PREDICT) {  /* the average */
+		switch (rpnp[rpi].op) {
+		case OP_PREDICT:
 		    if (count>0) {
 			val = sum/(double)count;
-		    } 
-		} else {
+		    }
+		    break;
+		case OP_PREDICTSIGMA:
 		    if (count>1) { /* the sigma case */
 			val=count*sum2-sum*sum;
 			if (val<0) {
@@ -892,6 +1036,29 @@ short rpn_calc(
 			    val=sqrt(val/((float)count*((float)count-1.0)));
 			}
 		    }
+		    break;
+		case OP_PREDICTPERC:
+		    if ((count>0) && extra) {
+		        /* sort the numbers */
+		        qsort(extra,count,sizeof(double),rpn_compare_double);
+			/* get the percentile selected */
+			double idxf = percentile * ((float)count-1.0);
+			if (percentile < 0) { /* take the next best */
+			    int idx = round(abs(idxf));
+			    val = extra[idx];
+			} else { /* interpolate */
+			    int idx = floor(idxf);
+			    double deltax = idxf - idx;
+			    val = extra[idx];
+			    if (deltax) { /* this check also handles the percentile=100 case */
+			      double deltay = extra[idx + 1] - extra[idx];
+			      val += deltay * deltax;
+			    }
+			}
+		    }
+		    break;
+		default: /* should not get here ... */
+		    break;
 		}
 		rpnstack->s[stptr] = val;
 	    }
@@ -961,6 +1128,110 @@ short rpn_calc(
             stackunderflow(0);
             rpnstack->s[stptr] = fabs(rpnstack->s[stptr]);
             break;
+        case OP_MEDIAN:
+            stackunderflow(0);
+            {
+                int elements = (int) rpnstack->s[stptr--];
+                int final_elements = elements;
+                double *element_ptr = rpnstack->s + stptr - elements + 1;
+                double *goodvals = element_ptr;
+                double *badvals = element_ptr + elements - 1;
+
+                stackunderflow(elements - 1);
+
+                /* move values to consolidate the non-NANs for sorting, keeping
+                 * track of how many NANs we encounter. */
+                while (goodvals < badvals) {
+                    if (isnan(*goodvals)) {
+                        *goodvals = *badvals--;
+                        --final_elements;
+                    } else {
+                        ++goodvals;
+                    }
+                }
+
+                stptr -= elements;
+                if (!final_elements) {
+                    /* no non-NAN elements; push NAN */
+                    rpnstack->s[++stptr] = DNAN;
+                } else {
+                    /* when goodvals and badvals meet, they might have met on a
+                     * NAN, which wouldn't decrease final_elements. so, check
+                     * that now. */
+                    if (isnan(*goodvals)) --final_elements;
+                    /* and finally, take the median of the remaining non-NAN
+                     * elements. */
+                    qsort(element_ptr, final_elements, sizeof(double),
+                          rpn_compare_double);
+                    if (final_elements % 2 == 1){
+                       rpnstack->s[++stptr] = element_ptr[ final_elements / 2 ];
+                    }
+                    else {
+                       rpnstack->s[++stptr] = 0.5 * ( element_ptr[ final_elements / 2 ] + element_ptr[ final_elements / 2 - 1 ] );
+                    }
+                }
+            }
+            break;
+        case OP_ROLL:
+            stackunderflow(1);
+            {
+                int step = (int) rpnstack->s[stptr--];
+                int base = (int) rpnstack->s[stptr--];
+                int i = base;
+                int j = i + step;
+                double *tmp_stack;
+                stackunderflow(base-1);
+                tmp_stack = (double *)malloc(sizeof(double)*base);
+                if(!tmp_stack) {
+                    rrd_set_error("RPN out of memory (allocating %i objects)",base);
+                    return -1;
+                }
+                memcpy(tmp_stack,rpnstack->s + stptr,(sizeof(double)*base));
+                while(i--) {
+                    j--;
+                    while(j<0) { j += base; }
+                    while(j>=base) { j -= base; }
+                    rpnstack->s[stptr-i] = tmp_stack[j];
+                }
+                free(tmp_stack);
+            }
+            break;
+        case OP_INDEX:
+            stackunderflow(0);
+            {
+                int i = (int) rpnstack->s[stptr];
+                stackunderflow(i);
+                rpnstack->s[stptr] = rpnstack->s[stptr - i];
+            }
+            break;
+        case OP_COPY:
+            {
+                int base = (int) rpnstack->s[stptr--];
+                int i = base;
+                stackunderflow(base - 1);
+                /* allocate or grow the stack */
+                while (stptr + base > rpnstack->dc_stacksize) {
+                    /* could move this to a separate function */
+                    rpnstack->dc_stacksize += rpnstack->dc_stackblock;
+                    rpnstack->s = (double*)rrd_realloc(rpnstack->s,
+                                      (rpnstack->dc_stacksize) *
+                                      sizeof(*(rpnstack->s)));
+                    if (rpnstack->s == NULL) {
+                        rrd_set_error("RPN stack overflow");
+                        return -1;
+                    }
+                }
+                while(i--) {
+                  stptr++;
+                  rpnstack->s[stptr] = rpnstack->s[stptr - base];
+                }
+            }
+            break;
+        case OP_DEPTH:
+            stptr++;
+            rpnstack->s[stptr] = stptr;
+            break;
+
         case OP_END:
             break;
         }
