@@ -1,5 +1,8 @@
+#ifndef RRD_TOOL_H_3853987DDF7E4709A5B5849E5A6204F4
+#define RRD_TOOL_H_3853987DDF7E4709A5B5849E5A6204F4
+
 /*****************************************************************************
- * RRDtool 1.4.9  Copyright by Tobi Oetiker, 1997-2014
+ * RRDtool 1.5.4, Copyright 1997-2015 by Tobi Oetiker
  *****************************************************************************
  * rrd_tool.h   Common Header File
  *****************************************************************************/
@@ -7,16 +10,7 @@
 extern    "C" {
 #endif
 
-#ifndef _RRD_TOOL_H
-#define _RRD_TOOL_H
-
-#if defined(WIN32) && !defined(__CYGWIN__) && !defined(__CYGWIN32__)
-#include "../win32/config.h"
-#else
-#ifdef HAVE_CONFIG_H
-#include "../rrd_config.h"
-#endif
-#endif
+#include "rrd_config.h"
 
 #include "rrd.h"
 
@@ -44,7 +38,7 @@ extern    "C" {
 #else
 
 /* unix-only includes */
-#if !defined(isnan) && !defined(HAVE_ISNAN)
+#if !defined isnan && !defined HAVE_ISNAN
     int       isnan(
     double value);
 #endif
@@ -84,7 +78,8 @@ extern    "C" {
 
     int       rrd_create_fn(
     const char *file_name,
-    rrd_t *rrd);
+    rrd_t *rrd,
+    int no_overwrite);
     int rrd_fetch_fn (const char *filename,
             enum cf_en cf_idx,
             time_t *start,
@@ -94,9 +89,18 @@ extern    "C" {
             char ***ds_namv,
             rrd_value_t **data);
 
+    int rrd_fetch_empty(
+        time_t *start,
+        time_t *end,
+        unsigned long *step,
+        unsigned long *ds_cnt,
+        char *ds_nam,
+        char ***ds_namv,
+        rrd_value_t **data);
+
 
 #ifdef HAVE_LIBDBI
-int rrd_fetch_fn_libdbi(const char *filename, enum cf_en cf_idx,
+    int rrd_fetch_fn_libdbi(const char *filename, enum cf_en cf_idx,
  			time_t *start,time_t *end,
  			unsigned long *step,
  			unsigned long *ds_cnt,
@@ -104,17 +108,42 @@ int rrd_fetch_fn_libdbi(const char *filename, enum cf_en cf_idx,
  			rrd_value_t **data);
 #endif
 
+typedef int (*rrd_fetch_cb_t)(
+    const char     *filename,  /* name of the rrd */
+    enum cf_en     cf_idx, /* consolidation function */
+    time_t         *start,
+    time_t         *end,       /* which time frame do you want ?
+                                * will be changed to represent reality */
+    unsigned long  *step,      /* which stepsize do you want? 
+                                * will be changed to represent reality */
+    unsigned long  *ds_cnt,    /* number of data sources in file */
+    char           ***ds_namv, /* names of data_sources */
+    rrd_value_t    **data      /* two dimensional array containing the data */
+);
+                                                                                                
+
+    int rrd_fetch_fn_cb(const char *filename, enum cf_en cf_idx,
+        time_t *start,time_t *end,
+        unsigned long *step,
+        unsigned long *ds_cnt,
+        char        ***ds_namv,
+        rrd_value_t **data);
+
+    int rrd_fetch_cb_register(rrd_fetch_cb_t cb);
+
+
 #define RRD_READONLY    (1<<0)
 #define RRD_READWRITE   (1<<1)
 #define RRD_CREAT       (1<<2)
 #define RRD_READAHEAD   (1<<3)
 #define RRD_COPY        (1<<4)
 #define RRD_EXCL        (1<<5)
+#define RRD_READVALUES  (1<<6)
 
     enum cf_en cf_conv(
     const char *string);
     enum dst_en dst_conv(
-    char *string);
+    const char *string);
     long      ds_match(
     rrd_t *rrd,
     char *ds_nam);
@@ -124,8 +153,10 @@ int rrd_fetch_fn_libdbi(const char *filename, enum cf_en cf_idx,
     char *a,
     char *b);
 
-#endif /* _RRD_TOOL_H */
+    const char *cf_to_string (enum cf_en cf);
 
 #ifdef  __cplusplus
 }
 #endif
+
+#endif /* RRD_TOOL_H */
