@@ -1,10 +1,8 @@
 /*****************************************************************************
- * RRDtool 1.4.9  Copyright by Tobi Oetiker, 1997-2014
+ * RRDtool 1.5.4, Copyright 1997-2015 by Tobi Oetiker
  *                Copyright by Florian Forster, 2008
  *****************************************************************************
  * rrd_update.c  RRD Update Function
- *****************************************************************************
- * $Id$
  *****************************************************************************/
 
 #include "rrd_tool.h"
@@ -16,6 +14,9 @@
 #endif
 
 #include <locale.h>
+#ifdef HAVE_STDINT_H
+#  include <stdint.h>
+#endif
 
 #include "rrd_hw.h"
 #include "rrd_rpncalc.h"
@@ -24,21 +25,19 @@
 #include "unused.h"
 
 #include "rrd_client.h"
+#include "rrd_update.h"
+#include <glib.h>
+
+#include "rrd_strtod.h"
 
 #if defined(_WIN32) && !defined(__CYGWIN__) && !defined(__CYGWIN32__)
+
 /*
  * WIN32 does not have gettimeofday	and struct timeval. This is a quick and dirty
  * replacement.
  */
 #include <sys/timeb.h>
 
-#ifndef __MINGW32__
-struct timeval {
-    time_t    tv_sec;   /* seconds */
-    long      tv_usec;  /* microseconds */
-};
-#endif
-
 struct __timezone {
     int       tz_minuteswest;   /* minutes W of Greenwich */
     int       tz_dsttime;   /* type of dst correction */
@@ -68,9 +67,10 @@ int       rrd_update_r(
     const char *tmplt,
     int argc,
     const char **argv);
-int       _rrd_update(
+int       _rrd_updatex(
     const char *filename,
     const char *tmplt,
+    int extra_flags,
     int argc,
     const char **argv,
     rrd_info_t *);
@@ -196,18 +196,6 @@ static int update_cdp_prep(
     rrd_value_t *seasonal_coef,
     int current_cf);
 
-static void update_cdp(
-    unival *scratch,
-    int current_cf,
-    rrd_value_t pdp_temp_val,
-    unsigned long rra_step_cnt,
-    unsigned long elapsed_pdp_st,
-    unsigned long start_pdp_offset,
-    unsigned long pdp_cnt,
-    rrd_value_t xff,
-    int i,
-    int ii);
-
 static void initialize_cdp_val(
     unival *scratch,
     int current_cf,
@@ -271,7 +259,7 @@ static int smooth_all_rras(
     rrd_file_t *rrd_file,
     unsigned long rra_begin);
 
-#ifndef HAVE_MMAP
+#if !defined(HAVE_MMAP) || defined(HAVE_LIBRADOS)
 static int write_changes_to_disk(
     rrd_t *rrd,
     rrd_file_t *rrd_file,
@@ -319,11 +307,13 @@ rrd_info_t *rrd_update_v(
     char **argv)
 {
     char     *tmplt = NULL;
+    int      extra_flags = 0;
     rrd_info_t *result = NULL;
     rrd_infoval_t rc;
     char *opt_daemon = NULL;
     struct option long_options[] = {
         {"template", required_argument, 0, 't'},
+        {"skip-past-updates",  no_argument, 0, 's'},
         {0, 0, 0, 0}
     };
 
@@ -335,7 +325,7 @@ rrd_info_t *rrd_update_v(
         int       option_index = 0;
         int       opt;
 
-        opt = getopt_long(argc, argv, "t:", long_options, &option_index);
+        opt = getopt_long(argc, argv, "st:", long_options, &option_index);
 
         if (opt == EOF)
             break;
@@ -352,7 +342,7 @@ rrd_info_t *rrd_update_v(
     }
 
     opt_daemon = getenv (ENV_RRDCACHED_ADDRESS);
-    if (opt_daemon != NULL) {
+    if (opt_daemon != NULL && ! strcmp(opt_daemon,"")) {
         rrd_set_error ("The \"%s\" environment variable is defined, "
                 "but \"%s\" cannot work with rrdcached. Either unset "
                 "the environment variable or use \"update\" instead.",
@@ -367,7 +357,7 @@ rrd_info_t *rrd_update_v(
     }
     rc.u_int = 0;
     result = rrd_info_push(NULL, sprintf_alloc("return_value"), RD_I_INT, rc);
-    rc.u_int = _rrd_update(argv[optind], tmplt,
+    rc.u_int = _rrd_updatex(argv[optind], tmplt,extra_flags,
                            argc - optind - 1,
                            (const char **) (argv + optind + 1), result);
     result->value.u_int = rc.u_int;
@@ -375,6 +365,315 @@ rrd_info_t *rrd_update_v(
     return result;
 }
 
+static char *rrd_get_file_template(const char *filename) /* {{{ */
+{
+	rrd_t rrd;
+	rrd_file_t *rrd_file;
+	unsigned int i;
+	size_t len=0;
+	char *filetemplate = NULL; //template is a key word in C++ , see http://msdn.microsoft.com/de-de/library/2e6a4at9.aspx
+
+	/* open file */
+	rrd_init(&rrd);
+	rrd_file = rrd_open(filename, &rrd, RRD_READONLY);
+	if (rrd_file == NULL)
+		goto err_free;
+
+	/* read the data from the rrd */
+	for (i = 0; i < rrd.stat_head->ds_cnt; i++) {
+		len += strlen(rrd.ds_def[i].ds_nam)+1;
+	}
+	/* now that we got it allocate memory */
+	filetemplate = (char *) malloc(len);
+	if (!filetemplate)
+		goto err_close;
+	filetemplate[0] = 0;
+
+	/* fill in for real */
+	for (i = 0; i < rrd.stat_head->ds_cnt; i++) {
+		if (i)
+			strcat(filetemplate,":");
+		strcat(filetemplate,rrd.ds_def[i].ds_nam);
+	}
+
+err_close:
+	rrd_close(rrd_file);
+err_free:
+	rrd_free(&rrd);
+	return (filetemplate);
+} /* }}} const char *rrd_get_file_template */
+
+
+/* the file-template-cache implementation */
+static GTree *rrd_file_template_cache = NULL;
+/* the neccesary functions for the gtree */
+static gint cache_compare_names (gconstpointer name1,
+				gconstpointer name2,
+				gpointer data)
+{
+	(void)(data); /* to avoid unused message */
+	return (strcmp((const char *)name1, (const char *)name2));
+}
+
+static void cache_destroy(gpointer data)
+{
+	free(data);
+}
+
+static const char *rrd_get_file_template_format(const char *filename) /* {{{ */
+{
+	char *format = NULL;
+	/* create rrd_file_template_cache  if needed */
+	if (!rrd_file_template_cache) {
+		rrd_file_template_cache = g_tree_new_full (
+			cache_compare_names,
+			NULL,
+			cache_destroy,
+			cache_destroy);
+		if (!rrd_file_template_cache)
+			return NULL;
+	}
+
+	/* fetch from cache */
+	format = (char *) g_tree_lookup(rrd_file_template_cache,
+					filename);
+	if (format)
+		return format;
+
+	/* fetch information from file */
+	format = rrd_get_file_template(filename);
+	if (!format)
+	        return NULL;
+
+	/* create copy of filename */
+	filename = strdup(filename);
+	if (!filename)
+		goto free_format;
+
+	/* and add object to tree */
+	g_tree_insert (rrd_file_template_cache,
+		       (char *)filename,
+		format);
+
+	return format;
+
+free_format:
+	free((void *)format);
+	return NULL;
+} /* }}} const char *rrd_get_file_template_format */
+
+static size_t _count_fields(const char *field)
+{
+	int c=1;
+	/* check for empty */
+	if (!field)
+		return 0;
+	if (!field[0])
+		return 0;
+	/* now count */
+	while((field = strchr(field, ':'))) {
+		c++;
+		field++;
+	}
+	return c;
+}
+
+static int _concat_field_n(char* string, const char *value, int field) {
+	const char *colon;
+	size_t len;
+	/* find the field */
+	while (field) {
+		value = strchr(value, ':');
+		if (!value)
+			return -1;
+		value++;
+		field--;
+	}
+
+	/* get the end of string and calculate length */
+	colon = strchr(value, ':');
+	if (colon) {
+		len = colon-value;
+	} else {
+		len = strlen(value);
+	}
+	/* now concat strings */
+	strncat(string, value, len);
+
+	/* return 1 field */
+	return 1;
+}
+
+static int _concat_field(
+	char *string,
+	const char* tpl,
+	const char* value,
+	const char* field)
+{
+	int fieldidx=0;
+	size_t len = 0;
+
+	/* get length */
+	char *colon = strchr((char *)field, ':');
+	if (colon) {
+		len=colon-field;
+	} else
+		len=strlen(field);
+
+	/* concat ":" */
+	strcat(string, ":");
+
+	/* find field in tpl */
+	while(tpl) {
+		/* check if it matches (including terminating 0 or :) */
+		if((strncmp(tpl, field, len)==0)) {
+			if ((tpl[len] == 0) || (tpl[len] == ':')) {
+				return _concat_field_n(
+					string, value, fieldidx+1);
+			}
+		}
+		/* increment tpl */
+		tpl = strchr(tpl, ':');
+		if (tpl)
+			tpl++;
+		/* and also increase the field index */
+		fieldidx++;
+	}
+
+	/* concat "U" */
+	strcat(string, "U");
+
+	/* and return "not found" */
+	return 0;
+}
+
+static char *rrd_map_template_to_values(const char *tpl,  /* {{{ */
+					const char *file_tpl,
+					const char *value)
+{
+	char *mapped           = NULL;
+	size_t fields_count    = 0;
+	size_t fields_tpl      = _count_fields(tpl);
+	size_t fields_file_tpl = _count_fields(file_tpl);
+	size_t fields_value    = _count_fields(value);
+	const char *ptr;
+	size_t len;
+	size_t i;
+	int ret;
+
+	/* check number of fields*/
+	if (fields_tpl != fields_value-1) {
+		rrd_set_error("rrd_map_template_to_values: "
+			"mismatch of number of fields in template (%zu) "
+			"with number of fields in values (%zu)",
+			fields_tpl,fields_value-1);
+		return NULL;
+	}
+	if (fields_tpl > fields_file_tpl) {
+		rrd_set_error("rrd_map_template_to_values: "
+			"number of fields in template (%zu) "
+			"bigger than number of fields in rrdfile (%zu)",
+			fields_tpl,fields_file_tpl);
+		return NULL;
+	}
+
+	/* now calculate effective length and allocate it */
+	len = strlen(value) /* length of the value */
+	        + 1 /* terminating null byte */
+  	        + (fields_file_tpl-fields_tpl) /* number of fields that we have
+						* more in the file_template
+						* compared to the given template
+						*/
+	        * 2 /* = strlen(":U") */;
+
+	mapped = (char *) malloc(len);
+	if (!mapped)
+		return NULL;
+	mapped[0] = 0;
+
+	/* first copy timestamp */
+	if (_concat_field_n(mapped,value,0)<0)
+		goto err;
+
+	/* now walk the list of fields from rrdtemplate*/
+	ptr = file_tpl;
+	for (i=0; i<fields_file_tpl; i++) {
+		ret = _concat_field(mapped, tpl, value, ptr);
+		if (ret < 0)
+			goto err;
+		fields_count += ret;
+
+		ptr = strchr(ptr, ':');
+		if (ptr)
+			ptr++;
+	}
+
+	/* check that we do not have a missmatch */
+	if (fields_count != fields_tpl) {
+		/* we could here more explicit,
+		 * by checking the missing fields
+		 */
+		rrd_set_error("rrd_map_template_to_values: "
+			"there are fields in template (%s) "
+			"that are not in the rrdfile (%s)",
+			tpl,file_tpl);
+		goto err;
+	}
+
+	/* return the mapped */
+	return mapped;
+err:
+	free(mapped);
+	return NULL;
+} /* }}} const char *rrd_map_template_to_values */
+
+static int rrd_template_update(const char *filename,  /* {{{ */
+			const char *tpl,
+			int values_num,
+			const char * const *values)
+{
+	int i;
+	int ret = -1;
+	char **mapped_values = NULL;
+	const char *file_format;
+
+	file_format= rrd_get_file_template_format(filename);
+	if (!file_format)
+		return -ENOMEM;
+
+
+	/* now start to map those fields */
+	mapped_values = (char **)calloc(values_num,sizeof(char*));
+	if (!mapped_values) {
+		rrd_set_error("rrd_template_update: "
+			" could not allocate memory");
+		goto error;
+	}
+
+	/* map the values */
+	for(i=0;i<values_num;i++) {
+		mapped_values[i] =
+			rrd_map_template_to_values(
+				tpl, file_format, values[i]);
+		if (!mapped_values[i])
+			goto error;
+	}
+
+	/* now call the real function */
+	ret = rrdc_update(filename, values_num,
+			(const char * const *) mapped_values);
+
+error:
+	/* free the temporary structures again */
+	if (mapped_values) {
+		for(i=0;i<values_num;i++)
+			free(mapped_values[i]);
+		free(mapped_values);
+	}
+
+	return ret;
+} /* }}} int rrd_template_update */
+
 int rrd_update(
     int argc,
     char **argv)
@@ -382,11 +681,13 @@ int rrd_update(
     struct option long_options[] = {
         {"template", required_argument, 0, 't'},
         {"daemon",   required_argument, 0, 'd'},
+        {"skip-past-updates",  no_argument, 0, 's'},
         {0, 0, 0, 0}
     };
     int       option_index = 0;
     int       opt;
-    char     *tmplt = NULL;
+    int       extra_flags = 0;
+    char      *tmplt = NULL;
     int       rc = -1;
     char     *opt_daemon = NULL;
 
@@ -394,7 +695,7 @@ int rrd_update(
     opterr = 0;         /* initialize getopt */
 
     while (1) {
-        opt = getopt_long(argc, argv, "t:d:", long_options, &option_index);
+        opt = getopt_long(argc, argv, "t:d:s", long_options, &option_index);
 
         if (opt == EOF)
             break;
@@ -404,6 +705,10 @@ int rrd_update(
             tmplt = strdup(optarg);
             break;
 
+        case 's':
+            extra_flags |= RRD_SKIP_PAST_UPDATES;
+            break;
+
         case 'd':
             if (opt_daemon != NULL)
                 free (opt_daemon);
@@ -435,26 +740,35 @@ int rrd_update(
         }
     }
 
-    if ((tmplt != NULL) && rrdc_is_connected(opt_daemon))
-    {
-        rrd_set_error("The caching daemon cannot be used together with "
-                "templates yet.");
-        goto out;
-    }
-
     if (! rrdc_is_connected(opt_daemon))
     {
-      rc = rrd_update_r(argv[optind], tmplt,
+      rc = rrd_updatex_r(argv[optind], tmplt,extra_flags,
                         argc - optind - 1, (const char **) (argv + optind + 1));
     }
     else /* we are connected */
     {
-        rc = rrdc_update (argv[optind], /* file */
-                          argc - optind - 1, /* values_num */
-                          (const char *const *) (argv + optind + 1)); /* values */
-        if (rc > 0)
-            rrd_set_error("Failed sending the values to rrdcached: %s",
-                          rrd_strerror (rc));
+	rrd_clear_error();
+	if (tmplt) {
+	        if (extra_flags != 0) {
+	            rrd_set_error("The caching daemon cannot be used together with "
+				  "templates and skip-past-updates yet.");
+		    goto out;
+		} else {
+ 		    rc = rrd_template_update(
+					     argv[optind], /* file */
+					     tmplt,
+					     argc - optind - 1, /* values_num */
+					     (const char *const *) (argv + optind + 1)); /* values */
+		}
+	} else
+		rc = rrdc_update (
+			argv[optind], /* file */
+			argc - optind - 1, /* values_num */
+			(const char *const *) (argv + optind + 1)); /* values */
+	if (rc > 0)
+		if (!rrd_test_error())
+			rrd_set_error("Failed sending the values to rrdcached: %s",
+				rrd_strerror (rc));
     }
 
   out:
@@ -477,25 +791,57 @@ int rrd_update_r(
     int argc,
     const char **argv)
 {
-    return _rrd_update(filename, tmplt, argc, argv, NULL);
+    return _rrd_updatex(filename, tmplt, 0, argc, argv, NULL);
 }
 
-int _rrd_update(
+int rrd_update_v_r(
     const char *filename,
     const char *tmplt,
     int argc,
     const char **argv,
     rrd_info_t * pcdp_summary)
 {
+    return _rrd_updatex(filename, tmplt, 0, argc, argv, pcdp_summary);
+}
+
+int rrd_updatex_r(
+    const char *filename,
+    const char *tmplt,
+    int extra_flags,
+    int argc,
+    const char **argv)
+{
+    return _rrd_updatex(filename, tmplt, extra_flags, argc, argv, NULL);
+}
+
+int rrd_updatex_v_r(
+    const char *filename,
+    const char *tmplt,
+    int extra_flags,
+    int argc,
+    const char **argv,
+    rrd_info_t * pcdp_summary)
+{
+    return _rrd_updatex(filename, tmplt, extra_flags, argc, argv, pcdp_summary);
+}
+
+int _rrd_updatex(
+    const char *filename,
+    const char *tmplt,
+    int extra_flags,
+    int argc,
+    const char **argv,
+    rrd_info_t * pcdp_summary)
+{
 
     int       arg_i = 2;
 
     unsigned long rra_begin;    /* byte pointer to the rra
                                  * area in the rrd file.  this
                                  * pointer never changes value */
-    rrd_value_t *pdp_new;   /* prepare the incoming data to be added 
+    rrd_value_t *pdp_new;   /* prepare the incoming data to be added
                              * to the existing entry */
-    rrd_value_t *pdp_temp;  /* prepare the pdp values to be added 
+    rrd_value_t *pdp_temp;  /* prepare the pdp values to be added
                              * to the cdp values */
 
     long     *tmpl_idx; /* index representing the settings
@@ -514,6 +860,7 @@ int _rrd_update(
     rrd_file_t *rrd_file;
     char     *arg_copy; /* for processing the argv */
     unsigned long *skip_update; /* RRAs to advance but not write */
+    int      process_ret;
 
     /* need at least 1 arguments: data. */
     if (argc < 1) {
@@ -553,11 +900,12 @@ int _rrd_update(
             rrd_set_error("failed duplication argv entry");
             break;
         }
-        if (process_arg(arg_copy, &rrd, rrd_file, rra_begin,
+        process_ret = process_arg(arg_copy, &rrd, rrd_file, rra_begin,
                         &current_time, &current_time_usec, pdp_temp, pdp_new,
                         rra_step_cnt, updvals, tmpl_idx, tmpl_cnt,
                         &pcdp_summary, version, skip_update,
-                        &schedule_smooth) == -1) {
+                        &schedule_smooth);
+        if ( ( process_ret == -1 ) || ( ! (extra_flags & RRD_SKIP_PAST_UPDATES) && process_ret == -2 ) ) {
             if (rrd_test_error()) { /* Should have error string always here */
                 char     *save_error;
 
@@ -570,6 +918,9 @@ int _rrd_update(
             free(arg_copy);
             break;
         }
+        if ( process_ret == -2 ){
+            rrd_clear_error();
+        }
         free(arg_copy);
     }
 
@@ -580,6 +931,10 @@ int _rrd_update(
     if (rrd_test_error()) {
         goto err_free_structures;
     }
+#ifdef HAVE_LIBRADOS
+    if (rrd_file->rados)
+      write_changes_to_disk(&rrd, rrd_file, version);
+#endif
 #ifndef HAVE_MMAP
     if (write_changes_to_disk(&rrd, rrd_file, version) == -1) {
         goto err_free_structures;
@@ -764,7 +1119,7 @@ static int parse_template(
  * Parse an update string, updates the primary data points (PDPs)
  * and consolidated data points (CDPs), and writes changes to the RRAs.
  *
- * Returns 0 on success, -1 on error.
+ * Returns 0 on success, -1 on error, -2 on time stamp error.
  */
 static int process_arg(
     char *step_start,
@@ -792,17 +1147,17 @@ static int process_arg(
     double    interval, pre_int, post_int;  /* interval between this and
                                              * the last run */
     unsigned long proc_pdp_cnt;
-
-    if (parse_ds(rrd, updvals, tmpl_idx, step_start, tmpl_cnt,
-                 current_time, current_time_usec, version) == -1) {
-        return -1;
+    int ds_ret;
+    if ((ds_ret = parse_ds(rrd, updvals, tmpl_idx, step_start, tmpl_cnt,
+                 current_time, current_time_usec, version)) != 0) {
+        return ds_ret;
     }
 
     interval = (double) (*current_time - rrd->live_head->last_up)
         + (double) ((long) *current_time_usec -
                     (long) rrd->live_head->last_up_usec) / 1e6f;
 
-    /* process the data sources and update the pdp_prep 
+    /* process the data sources and update the pdp_prep
      * area accordingly */
     if (update_pdp_prep(rrd, updvals, pdp_new, interval) == -1) {
         return -1;
@@ -911,7 +1266,7 @@ static int parse_ds(
             else {
                 rrd_set_error("found extra data on update argument: %s",p+1);
                 return -1;
-            }                
+            }
         }
     }
 
@@ -921,16 +1276,13 @@ static int parse_ds(
         return -1;
     }
 
-    if (get_time_from_reading(rrd, timesyntax, updvals,
+    return get_time_from_reading(rrd, timesyntax, updvals,
                               current_time, current_time_usec,
-                              version) == -1) {
-        return -1;
-    }
-    return 0;
+                              version);
 }
 
 /*
- * Parse the time in a DS string, store it in current_time and 
+ * Parse the time in a DS string, store it in current_time and
  * current_time_usec and verify that it's later than the last
  * update for this DS.
  *
@@ -946,7 +1298,6 @@ static int get_time_from_reading(
 {
     double    tmp;
     char     *parsetime_error = NULL;
-    char     *old_locale;
     rrd_time_value_t ds_tv;
     struct timeval tmp_time;    /* used for time conversion */
 
@@ -970,15 +1321,9 @@ static int get_time_from_reading(
         *current_time = tmp_time.tv_sec;
         *current_time_usec = tmp_time.tv_usec;
     } else {
-        old_locale = setlocale(LC_NUMERIC, "C");
-        errno = 0;
-        tmp = strtod(updvals[0], 0);
-        if (errno > 0) {
-            rrd_set_error("converting '%s' to float: %s",
-                updvals[0], rrd_strerror(errno));
+        if ( rrd_strtodbl( updvals[0], NULL, &tmp, "error while parsing time in get_time_from_reading") != 2) {
             return -1;
         };
-        setlocale(LC_NUMERIC, old_locale);
         if (tmp < 0.0){
             gettimeofday(&tmp_time, 0);
             tmp = (double)tmp_time.tv_sec + (double)tmp_time.tv_usec * 1e-6f + tmp;
@@ -997,7 +1342,7 @@ static int get_time_from_reading(
         rrd_set_error("illegal attempt to update using time %ld when "
                       "last update time is %ld (minimum one second step)",
                       *current_time, rrd->live_head->last_up);
-        return -1;
+        return -2;
     }
     return 0;
 }
@@ -1016,9 +1361,7 @@ static int update_pdp_prep(
 {
     unsigned long ds_idx;
     int       ii;
-    char     *endptr;   /* used in the conversion */
-    double    rate;
-    char     *old_locale;
+    double    rate, newval, oldval;
     enum dst_en dst_idx;
 
     for (ds_idx = 0; ds_idx < rrd->stat_head->ds_cnt; ds_idx++) {
@@ -1072,7 +1415,7 @@ static int update_pdp_prep(
                          * ... are there any others in SNMP land?
                          */
                         if (pdp_new[ds_idx] < (double) 0.0)
-                            pdp_new[ds_idx] += (double) 4294967296.0;   /* 2^32 */
+                            pdp_new[ds_idx] += (double) 4294967295.0;   /* 2^32-1 */
                         if (pdp_new[ds_idx] < (double) 0.0)
                             pdp_new[ds_idx] += (double) 18446744069414584320.0; /* 2^64-2^32 */
                     }
@@ -1082,42 +1425,54 @@ static int update_pdp_prep(
                 }
                 break;
             case DST_ABSOLUTE:
-                old_locale = setlocale(LC_NUMERIC, "C");
-                errno = 0;
-                pdp_new[ds_idx] = strtod(updvals[ds_idx + 1], &endptr);
-                if (errno > 0) {
-                    rrd_set_error("converting '%s' to float: %s",
-                                  updvals[ds_idx + 1], rrd_strerror(errno));
-                    return -1;
-                };
-                setlocale(LC_NUMERIC, old_locale);
-                if (endptr[0] != '\0') {
-                    rrd_set_error
-                        ("conversion of '%s' to float not complete: tail '%s'",
-                         updvals[ds_idx + 1], endptr);
+                if( rrd_strtodbl(updvals[ds_idx + 1], NULL, &newval, "Function update_pdp_prep, case DST_ABSOLUTE" ) != 2 ) {
                     return -1;
                 }
+                pdp_new[ds_idx] = newval;
                 rate = pdp_new[ds_idx] / interval;
                 break;
             case DST_GAUGE:
-                old_locale = setlocale(LC_NUMERIC, "C");
-                errno = 0;
-                pdp_new[ds_idx] =
-                    strtod(updvals[ds_idx + 1], &endptr) * interval;
-                if (errno) {
-                    rrd_set_error("converting '%s' to float: %s",
-                                  updvals[ds_idx + 1], rrd_strerror(errno));
-                    return -1;
-                };
-                setlocale(LC_NUMERIC, old_locale);
-                if (endptr[0] != '\0') {
-                    rrd_set_error
-                        ("conversion of '%s' to float not complete: tail '%s'",
-                         updvals[ds_idx + 1], endptr);
+                if( rrd_strtodbl( updvals[ds_idx + 1], NULL, &newval, "Function update_pdp_prep, case DST_GAUGE") != 2 ) {
                     return -1;
                 }
-                rate = pdp_new[ds_idx] / interval;
+                pdp_new[ds_idx] = newval * interval;
+                rate = newval;
                 break;
+            case DST_DCOUNTER:
+            case DST_DDERIVE:
+                if (rrd->pdp_prep[ds_idx].last_ds[0] != 'U') {
+                    const char *e_msg;
+
+                    if (dst_idx == DST_DCOUNTER) {
+                        e_msg = "Function update_pdp_prep, case DST_DCOUNTER";
+                    } else {
+                        e_msg = "Function update_pdp_prep, case DST_DDERIVE";
+                    }
+                    if (rrd_strtodbl(updvals[ds_idx + 1], NULL, &newval, e_msg) != 2) {
+                        return -1;
+                    }
+                    if (rrd_strtodbl(rrd->pdp_prep[ds_idx].last_ds, NULL, &oldval, e_msg) != 2) {
+                        return -1;
+                    }
+                    if (dst_idx == DST_DCOUNTER) {
+                        /*
+                         * DST_DCOUNTER is always signed, so it can count either up,
+                         * or down, but not both at the same time. Changing direction
+                         * considered a "reset".
+                         */
+                        if ((newval > 0 && oldval > newval) ||
+                          (newval < 0 && newval > oldval)) {
+                            /* Counter reset detected */
+                            pdp_new[ds_idx] = DNAN;
+                            break;
+                        }
+                    }
+                    pdp_new[ds_idx] = newval - oldval;
+                    rate = pdp_new[ds_idx] / interval;
+                } else {
+                    pdp_new[ds_idx] = DNAN;
+                 }
+                 break;
             default:
                 rrd_set_error("rrd contains unknown DS type : '%s'",
                               rrd->ds_def[ds_idx].dst);
@@ -1176,7 +1531,7 @@ static int calculate_elapsed_steps(
     unsigned long proc_pdp_st;  /* which pdp_st was the last to be processed */
     unsigned long occu_pdp_st;  /* when was the pdp_st before the last update
                                  * time */
-    unsigned long proc_pdp_age; /* how old was the data in the pdp prep area 
+    unsigned long proc_pdp_age; /* how old was the data in the pdp prep area
                                  * when it was last updated */
     unsigned long occu_pdp_age; /* how long ago was the last pdp_step time */
 
@@ -1367,13 +1722,18 @@ static int process_pdp_st(
                 rpnp[i].op = OP_NUMBER;
                 rpnp[i].val = pdp_temp[rpnp[i].ptr];
             }
+	    /* just in case */
+	    rpnp[i].extra = NULL;
+	    rpnp[i].free_extra = NULL;
         }
         /* run the rpn calculator */
-        if (rpn_calc(rpnp, &rpnstack, 0, pdp_temp, ds_idx) == -1) {
+        if (rpn_calc(rpnp, &rpnstack, 0, pdp_temp, ds_idx,rrd->stat_head->pdp_step) == -1) {
+	    rpnp_freeextra(rpnp);
             free(rpnp);
             rpnstack_free(&rpnstack);
             return -1;
         }
+	rpnp_freeextra(rpnp);
         free(rpnp);
     }
 
@@ -1430,8 +1790,9 @@ static int update_all_cdp_prep(
             proc_pdp_cnt % rrd->rra_def[rra_idx].pdp_cnt;
         skip_update[rra_idx] = 0;
         if (start_pdp_offset <= elapsed_pdp_st) {
-            rra_step_cnt[rra_idx] = (elapsed_pdp_st - start_pdp_offset) /
-                rrd->rra_def[rra_idx].pdp_cnt + 1;
+            rra_step_cnt[rra_idx] = min((elapsed_pdp_st - start_pdp_offset) /
+                rrd->rra_def[rra_idx].pdp_cnt + 1,
+                rrd->rra_def[rra_idx].row_cnt);
         } else {
             rra_step_cnt[rra_idx] = 0;
         }
@@ -1476,7 +1837,7 @@ static int update_all_cdp_prep(
     return 0;
 }
 
-/* 
+/*
  * Are we due for a smooth? Also increments our position in the burn-in cycle.
  */
 static int do_schedule_smooth(
@@ -1565,7 +1926,7 @@ static int update_cdp_prep(
  * Given the new reading (pdp_temp_val), update or initialize the CDP value,
  * primary value, secondary value, and # of unknowns.
  */
-static void update_cdp(
+void update_cdp(
     unival *scratch,
     int current_cf,
     rrd_value_t pdp_temp_val,
@@ -1658,7 +2019,7 @@ static void initialize_cdp_val(
             (cum_val + cur_val * start_pdp_offset) /
             (pdp_cnt - scratch[CDP_unkn_pdp_cnt].u_cnt);
         break;
-    case CF_MAXIMUM: 
+    case CF_MAXIMUM:
         cum_val = IFDNAN(scratch[CDP_val].u_val, -DINF);
         cur_val = IFDNAN(pdp_temp_val, -DINF);
 
@@ -1777,16 +2138,16 @@ static rrd_value_t initialize_carry_over(
             return 0;
         default:
             return DNAN;
-        }        
-    } 
+        }
+    }
     else {
         switch (current_cf) {
         case CF_AVERAGE:
             return pdp_temp_val *  pdp_into_cdp_cnt ;
         default:
             return pdp_temp_val;
-        }        
-    }        
+        }
+    }
 }
 
 /*
@@ -1885,7 +2246,7 @@ static int update_aberrant_cdps(
     return 0;
 }
 
-/* 
+/*
  * Move sequentially through the file, writing one RRA at a time.  Note this
  * architecture divorces the computation of CDP with flushing updated RRA
  * entries to disk.
@@ -1906,7 +2267,7 @@ static int write_to_rras(
     time_t    rra_time = 0; /* time of update for a RRA */
 
     unsigned long ds_cnt = rrd->stat_head->ds_cnt;
-    
+
     /* Ready to write to disk */
     rra_start = rra_begin;
 
@@ -2001,7 +2362,7 @@ static int write_RRA_row(
             /* append info to the return hash */
             *pcdp_summary = rrd_info_push(*pcdp_summary,
                                           sprintf_alloc
-                                          ("[%lli]RRA[%s][%lu]DS[%s]", 
+                                          ("[%lli]RRA[%s][%lu]DS[%s]",
                                            (long long)rra_time,
                                            rrd->rra_def[rra_idx].cf_nam,
                                            rrd->rra_def[rra_idx].pdp_cnt,
@@ -2048,7 +2409,7 @@ static int smooth_all_rras(
     return 0;
 }
 
-#ifndef HAVE_MMAP
+#if !defined(HAVE_MMAP) || defined(HAVE_LIBRADOS)
 /*
  * Flush changes to disk (unless we're using mmap)
  *
