@@ -1,5 +1,8 @@
+#ifndef RRD_XPORT_H_C5A7EBAF331140D38C241326B81F357D
+#define RRD_XPORT_H_C5A7EBAF331140D38C241326B81F357D
+
 /****************************************************************************
- * RRDtool 1.4.9  Copyright by Tobi Oetiker, 1997-2014
+ * RRDtool 1.5.4, Copyright 1997-2015 by Tobi Oetiker
  ****************************************************************************
  * rrd_xport.h  contains XML related constants
  ****************************************************************************/
@@ -7,9 +10,6 @@
 extern    "C" {
 #endif
 
-#ifndef _RRD_XPORT_H
-#define _RRD_XPORT_H
-
 #define XML_ENCODING     "ISO-8859-1"
 #define ROOT_TAG         "xport"
 #define META_TAG         "meta"
@@ -25,10 +25,8 @@ extern    "C" {
 #define COL_TIME_TAG     "t"
 #define COL_DATA_TAG     "v"
 
-
-#endif
-
-
 #ifdef  __cplusplus
 }
 #endif
+
+#endif
